header {
package edu.usfca.syndiag;
}

/** A tree grammar describing output of antlr.g's AST representing
 *  ANTLR grammars.
 *
 *  Terence Parr
 *  University of San Francisco
 *  August 19, 2003
 */
class AdjustDimension extends TreeParser;

options {
	importVocab = ANTLR;
    codeGenBitsetTestThreshold=999;
}

{
public static int width = 600;
public static int altWidth = 300;
public GrammarAST root = null;
public String ruleName = "";

/**
* To set Width
**/
public void setWidth(int w) {
  width = w;
}

/**
* To set Alternative Width
**/
public void setAltWidth(int w) {
  altWidth = w;
}

public GrammarAST createRuleAST(String name){
	GrammarAST ruleName_AST = new GrammarAST();
	ruleName_AST.initialize(RULE_REF, name);
	GrammarAST block_AST = new GrammarAST();
    block_AST.initialize(BLOCK, "BLOCK");
    GrammarAST alt_AST = new GrammarAST();
    alt_AST.initialize(ALT, "ALT");
    block_AST.setFirstChild(alt_AST);
	AST eor = astFactory.create(EOR,"<end-of-rule>");
	GrammarAST rule_AST = new GrammarAST();
	rule_AST.initialize(RULE, "rule");
	rule_AST.setFirstChild(ruleName_AST);
	rule_AST.getFirstChild().setNextSibling(block_AST);
	rule_AST.getFirstChild().getNextSibling().setNextSibling(eor);
	return rule_AST;
}

}
grammar
    :   classDef // ignore 2nd (lexer) grammar for now
    ;

classDef
    :   #( PARSER rules )
    |   #( TREE_PARSER rules )
    |   #( LEXER rules )
    ;

rules
    :   ( r:rule {  AST t = r.getFirstChild().getNextSibling();
    				SetDimension sd = new SetDimension();
    				
					if(t.getNumberOfChildren() == 1){
					    AST t1 = t.getFirstChild();
					    // assert t1 is an "ALT"
					    int currX = ((GrammarAST)t1.getFirstChild()).getWidth()+((GrammarAST)t1.getFirstChild()).deltaX;
					    GrammarAST begin = null;
					    AST ndx = r;
					    GrammarAST rule_AST = null;
					    int count = 0;
					    String name;
					    for(GrammarAST t2 = (GrammarAST)t1.getFirstChild(); t2.getNextSibling()!=null; t2=(GrammarAST)t2.getNextSibling()){
					    	currX += ((GrammarAST)t2.getNextSibling()).getWidth() + ((GrammarAST)t2.getNextSibling()).deltaX;
					    	if((currX > width) && 
					    	   ((t2.getNextSibling() != t2.getLastSibling()) || 
					    	   	((t2.getNextSibling() == t2.getLastSibling()) &&
					    	   	 (((GrammarAST)t2.getNextSibling()).getWidth() > 200)
					    	   	)
					    	   )
					    	  ){
					    		currX = ((GrammarAST)t2.getNextSibling()).getWidth() + ((GrammarAST)t2.getNextSibling()).deltaX;;
					    		if(begin!=null){
					    			name = r.getFirstChild().getText() + count;
			                		rule_AST = createRuleAST(name);
			                		rule_AST.getFirstChild().getNextSibling().getFirstChild().setFirstChild(begin.getNextSibling());
			                		GrammarAST newLine = new GrammarAST();
			                		newLine.initialize(RULE_REF, name);
			    				    begin.setNextSibling(newLine);
									rule_AST.setNextSibling(ndx.getNextSibling());
									ndx.setNextSibling(rule_AST);
									ndx = rule_AST;
									rule_AST.setSignal(1);
					    		}
					    		begin = t2;
					    		count++;
					    	}
					    }
					    if( begin!=null ){
					    	name = r.getFirstChild().getText() + count;
	                		rule_AST = createRuleAST(name);
	                		rule_AST.getFirstChild().getNextSibling().getFirstChild().setFirstChild(begin.getNextSibling());
	    				    GrammarAST newLine = new GrammarAST();
	    				    newLine.initialize(RULE_REF, name);
			    			begin.setNextSibling(newLine);
							rule_AST.setNextSibling(ndx.getNextSibling());
							ndx.setNextSibling(rule_AST);						
							rule_AST.setSignal(1);
						}
						sd.rule(root);
                    }
                 } 
    	)+
    ;

rule
	{GrammarAST rl = (GrammarAST)#rule;
	 root = rl;
	}
    :   #( RULE r:RULE_REF
    				{ruleName = r.getText();
    				}
    			 #(BLOCK block[rl, "rule"]) EOR )
    ;

block [GrammarAST r, String altName]
	{int count = 0;
	 AST ndx = root;
	 int i = 0;
	 SetDimension sd = new SetDimension();
    }
    :   ( a:alternative 
    		{if (r != null){
    			if (r.getType() == RULE){
    		 		i = r.getFirstChild().getNextSibling().getNumberOfChildren();
    		 	}else{
    		 		i = r.getFirstChild().getNumberOfChildren();
    		 	}
    		 }
    		 if (i > 1){
	    		 int w = ((GrammarAST)a).getWidth();
	    		 GrammarAST rule_AST = null;
	    		 GrammarAST alt_AST = new GrammarAST();
	    		 String name;
	    		 if (w > altWidth) {
	    		 	// create rule AST
	    		 	name = ruleName + "_" + altName + "_ALT" + count;
	    		 	rule_AST = createRuleAST(name);
	    		 	rule_AST.getFirstChild().getNextSibling().getFirstChild().setFirstChild(a.getFirstChild());
	    		 	alt_AST.initialize(RULE_REF, name);
	    		 	a.setFirstChild(alt_AST);
					rule_AST.setNextSibling(ndx.getNextSibling());
					ndx.setNextSibling(rule_AST);
					ndx = rule_AST;
					rule_AST.setSignal(1);
					count ++;
				    sd.rule(root);
	    		 }
	    		}
    		}
    	)+
    ;

alternative
    :   #( ALT (element)+ )
    ;

element
	{GrammarAST e = (GrammarAST)#element;}
    :   atom
    |   #(NOT element) 
    |   #(TOKEN_RANGE atom atom)
    |   #(CHAR_RANGE CHAR_LITERAL CHAR_LITERAL)
    |   ebnf
    |   tree
    |   #(SYNPRED block[e, "ELT"] )
    |   ACTION
    |   SEMPRED
    |   EPSILON
    ;

ebnf
	{GrammarAST e = (GrammarAST)#ebnf;}
	:	#( BLOCK block[e, "B"] )
    |   #( OPTIONAL block[e, "O"] )
    |   #( CLOSURE block[e, "C"] )
    |   #( POSITIVE_CLOSURE block[e, "PC"] )
    ;

tree:   #(TREE_BEGIN atom (element)* )
    ;

atom
    :   RULE_REF
    |   TOKEN_REF
    |   CHAR_LITERAL
    |   STRING_LITERAL
    |   WILD_CARD
    ;


/*rule
    :   #( RULE r:RULE_REF #(BLOCK block) EOR )
	    {System.out.println("in adjustDim.g " + r.getText());}
    ;

block
    :   ( a:alternative )+
    ;

alternative
    :   #( ALT (element)+ )
    ;

element
    :   atom
    |   #(NOT element) 
    |   #(TOKEN_RANGE atom atom)
    |   #(CHAR_RANGE CHAR_LITERAL CHAR_LITERAL)
    |   ebnf
    |   tree
    |   #( SYNPRED block )
    |   ACTION
    |   SEMPRED
    |   EPSILON
    ;

ebnf:   #( BLOCK block )
    |   #( OPTIONAL block )
    |   #( CLOSURE block )
    |   #( POSITIVE_CLOSURE block )
    ;

tree:   #(TREE_BEGIN atom (element)* )
    ;

atom
    :   RULE_REF
    |   TOKEN_REF
    |   CHAR_LITERAL
    |   STRING_LITERAL
    |   WILD_CARD
    ;

*/
