package net.mark_malakanov.sdg2;


/*
 * [The "BSD licence"]
 * Copyright (c) Mark Malakanov 2004
 * All rights reserved.
 *
 * $Header: d:\\cvsroot/SDG2/src/net/mark_malakanov/sdg2/DrawDemo.java,v 1.4 2005/01/29 17:23:37 Mark Exp $
 *
 * Modification Log
 * $Log: DrawDemo.java,v $
 * Revision 1.4  2005/01/29 17:23:37  Mark
 * changed rectangles to be filled transpared ("none") when save to SVG.
 *
 * Revision 1.3  2005/01/03 13:16:47  Mark
 * minor
 *
 * Revision 1.2  2004/12/28 07:47:47  Mark
 * CVS keywords added
 *
 *
 */
import antlr.*;

import antlr.collections.*;

import antlr.debug.misc.*;

import edu.usfca.syndiag.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;


public class DrawDemo extends JPanel {
    AST t = null;
    SwingEngine swing;
    DefaultSyntaxDiagramGenerator diagramGen;
    Draw draw;
    
    public void init() {
        setBackground(Color.white);
        setForeground(Color.black);
        swing = new SwingEngine();
        diagramGen = new CanonicSyntaxDiagramGenerator(swing);
        draw = new Draw();
        draw.setDiagGen(diagramGen);
    }

    public void setDiagramStyle(String dg) {
      if (dg.equalsIgnoreCase("Heretical")) {
        diagramGen = new DefaultSyntaxDiagramGenerator(swing);
        draw.setDiagGen(diagramGen);
      } else if (dg.equalsIgnoreCase("Canonical")) {
        diagramGen = new CanonicSyntaxDiagramGenerator(swing);
        draw.setDiagGen(diagramGen);
      }
    }

    public void setAST(AST t) {
        this.t = t;
    }

    public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        Dimension d = getSize();
        g2.setBackground(getBackground());
        g2.setPaint(getBackground());
        g2.fillRect(0, 0, d.width, d.height);
        g2.setPaint(getForeground());
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);

        if (swing == null) { return; }
        swing.setGraphics2D(g2);

        
        if (t == null) {
            return;
        }

        try {
            draw.grammar(t);
        } catch (Exception e) {
            e.printStackTrace();
        }

        int width = ((GrammarAST) t).getWidth();
        int height = ((GrammarAST) t).getHeight();
        setPreferredSize(new Dimension(width, height));
        revalidate();
    }

    public void paintSvg(String fileName) {
        int width = ((GrammarAST) t).getWidth();
        int height = ((GrammarAST) t).getHeight();

        SvgEngine svg = new SvgEngine();
        svg.setColor("black");
        svg.setBgColor("none");

        svg.setWriter(fileName);
        svg.beginDraw(new Rectangle(0, 0, width, height));

        DefaultSyntaxDiagramGenerator diagramGen = new DefaultSyntaxDiagramGenerator(svg);
        Draw draw = new Draw();
        draw.setDiagGen(diagramGen);

        if (t == null) {
            return;
        }

        try {
            draw.grammar(t);
        } catch (Exception e) {
            e.printStackTrace();
        }

        svg.endDraw();
    }
}
